import { defineNuxtModule } from '@nuxt/kit';
import { log } from 'logzy';

const generateLocalHead = (local, nuxt) => {
  let fontFace = "";
  let classes = "";
  let root = "";
  let variables = "";
  for (const source of local) {
    const options = {
      weight: "400",
      display: "optional",
      style: "normal",
      ...source
    };
    const [, format] = options.src.split(/\.(?=[^.]+$)/);
    nuxt.options.app.head.link?.push({
      rel: "preload",
      as: "font",
      crossorigin: "anonymous",
      type: `font/${format}`,
      href: options.src
    });
    const fontFallback = options.fallback ? `,${options.fallback}` : "";
    const fontFamily = `font-family:"${options.family}";`;
    const fontWeight = `font-weight:${options.weight};`;
    const fontDisplay = `font-display:${options.display};`;
    const fontStyle = `font-style:${options.style};`;
    const fontSrc = `src:url('${options.src}') format('${format}');`;
    if (options.class)
      classes += `.${options.class}{font-family:"${options.family}"${fontFallback};}`;
    if (options.variable)
      variables += `--${options.variable}:"${options.family}"${fontFallback};`;
    fontFace += `@font-face{${fontFamily}${fontWeight}${fontDisplay}${fontStyle}${fontSrc}}`;
  }
  if (variables)
    root += `:root{${variables}}`;
  const styles = `${fontFace}${classes}${root}`;
  return nuxt.options.app.head.style?.push({ children: styles });
};
const generateExternalHead = (external, nuxt) => {
  let google = false;
  let typekit = false;
  let classes = "";
  let root = "";
  let variables = "";
  for (const source of external) {
    const options = {
      ...source
    };
    if (options.src.includes("google") && !google) {
      google = true;
      nuxt.options.app.head.link?.push({
        rel: "preconnect",
        href: "https://fonts.googleapis.com"
      });
      nuxt.options.app.head.link?.push({
        rel: "preconnect",
        crossorigin: "anonymous",
        href: "https://fonts.gstatic.com"
      });
    }
    if (options.src.includes("typekit") && !typekit) {
      typekit = true;
      nuxt.options.app.head.link?.push({
        rel: "preconnect",
        crossorigin: "anonymous",
        href: "https://use.typekit.net"
      });
    }
    nuxt.options.app.head.link?.push({
      rel: "preload",
      as: "style",
      href: options.src
    });
    nuxt.options.app.head.link?.push({
      rel: "stylesheet",
      href: options.src
    });
    const fontFallback = options.fallback ? `,${options.fallback}` : "";
    if (options.class)
      classes += `.${options.class}{font-family:"${options.family}"${fontFallback};}`;
    if (options.variable)
      variables += `--${options.variable}:"${options.family}"${fontFallback};`;
  }
  if (variables)
    root += `:root{${variables}}`;
  const styles = `${classes}${root}`;
  if (styles)
    return nuxt.options.app.head.style?.push({ children: styles });
};

const module = defineNuxtModule({
  meta: {
    name: "nuxt-font-loader",
    configKey: "fontLoader",
    compatibility: {
      nuxt: ">=3.0.0"
    }
  },
  defaults: {
    logs: true
  },
  setup(options, nuxt) {
    const { local, external, logs } = options;
    if (!local && !external && logs) {
      const warnMessage = `> nuxt-font-loader \u2014 The module is enabled but not configured. Set new font sources via module options.`;
      log(["magenta", "bold"], warnMessage);
    }
    if (local)
      generateLocalHead(local, nuxt);
    if (external)
      generateExternalHead(external, nuxt);
  }
});

export { module as default };
